require_dependency 'fullcalendar_engine_user/application_controller'

module FullcalendarEngineUser
  class EventsController < ApplicationController

    layout FullcalendarEngineUser::Configuration['layout'] || 'application'

    before_filter :load_event, only: [:edit, :update, :destroy, :move, :resize]
    before_filter :determine_event_type, only: :create

    def create
      if @event.save
	@event.update_column(:user_id,current_user.id)
        @event.save
        render nothing: true
      else
        render text: @event.errors.full_messages.to_sentence, status: 422
      end
    end

    def new
      respond_to do |format|
        format.js
      end
    end

    def get_events
      start_time = Time.at(params[:start].to_i).to_formatted_s(:db)
      end_time   = Time.at(params[:end].to_i).to_formatted_s(:db)

      if current_user.show_all == true and current_user.show_user == 0
      @allusers= []
      @ideventseries = []
      @usersall = User.all
      @usersall.each do |usersall|
      @allusers << usersall[:id] 
      end
      @eventseries = EventSeries.where("user_id in (?)", @allusers)
      @eventseries.each do |events|
      @ideventseries << events[:id] 
      end
      
      @events = Event.where('
                  ((starttime >= :start_time and endtime <= :end_time) or
                  (starttime >= :start_time and endtime > :end_time and starttime <= :end_time) or
                  (starttime <= :start_time and endtime >= :start_time and endtime <= :end_time) or
                  (starttime <= :start_time and endtime > :end_time)) and ((user_id in (:user_id)) or (event_series_id in (:massiv)) )',
                  start_time: start_time, end_time: end_time, user_id: @allusers.map, massiv: @ideventseries.map)
      elsif current_user.show_all == true and current_user.show_user > 0
      @ideventseries = []
      @eventseries = EventSeries.where("user_id = (?)", current_user.show_user)
      @eventseries.each do |events|
      @ideventseries << events[:id] 
      end
      @events = Event.where('
                  ((starttime >= :start_time and endtime <= :end_time) or
                  (starttime >= :start_time and endtime > :end_time and starttime <= :end_time) or
                  (starttime <= :start_time and endtime >= :start_time and endtime <= :end_time) or
                  (starttime <= :start_time and endtime > :end_time)) and ((user_id = :user_id) or (event_series_id in (:massiv)) )',
                  start_time: start_time, end_time: end_time, user_id: current_user.show_user, massiv: @ideventseries.map)
      else      
      @ideventseries = []
      @eventseries = EventSeries.where("user_id = (?)", current_user.id)
      @eventseries.each do |events|
      @ideventseries << events[:id] 
      end
      @events = Event.where('
                  ((starttime >= :start_time and endtime <= :end_time) or
                  (starttime >= :start_time and endtime > :end_time and starttime <= :end_time) or
                  (starttime <= :start_time and endtime >= :start_time and endtime <= :end_time) or
                  (starttime <= :start_time and endtime > :end_time)) and ((user_id = :user_id) or (event_series_id in (:massiv)) )',
                  start_time: start_time, end_time: end_time, user_id: current_user.id, massiv: @ideventseries.map)
      end
    
      events = []
      @events.each do |event|
        events << { id: event.id,
                    title: event.title,
                    description: event.description || '', 
                    start: event.starttime.iso8601,
                    end: event.endtime.iso8601,
                    allDay: event.all_day,
                    recurring: (event.event_series_id) ? true : false }
      end
      render json: events.to_json
    end

    def move
      if current_user.show_all == false and current_user.show_user == 0
        if @event
          @event.starttime = make_time_from_minute_and_day_delta(@event.starttime)
          @event.endtime   = make_time_from_minute_and_day_delta(@event.endtime)
          @event.all_day   = params[:all_day]
          @event.save
        end
      end
      render nothing: true
    end

    def resize
      if current_user.show_all == false and current_user.show_user == 0
        if @event
          @event.endtime = make_time_from_minute_and_day_delta(@event.endtime)
          @event.save
        end
      end    
      render nothing: true
    end

    def edit
      render json: { form: render_to_string(partial: 'edit_form') } 
    end

    def update
      if current_user.show_all == false and current_user.show_user == 0
        case params[:event][:commit_button]
        when 'Update All Occurrence'
          @events = @event.event_series.events
          @event.update_events(@events, event_params)
        when 'Update All Following Occurrence'
          @events = @event.event_series.events.where('starttime > :start_time', 
                                                   start_time: @event.starttime.to_formatted_s(:db)).to_a
          @event.update_events(@events, event_params)
        else
          @event.attributes = event_params
          @event.save
        end 
      end
      render nothing: true
    end

    def destroy
      if current_user.show_all == false and current_user.show_user == 0
        case params[:delete_all]
        when 'true'
          @event.event_series.destroy
        when 'future'
          @events = @event.event_series.events.where('starttime > :start_time',
                                                   start_time: @event.starttime.to_formatted_s(:db)).to_a
          @event.event_series.events.delete(@events)
        else
        @event.destroy
      end
      end
      render nothing: true
    end

    private

    def load_event
      @event = Event.where(:id => params[:id]).first
      unless @event
        render json: { message: "Event Not Found.."}, status: 404 and return
      end
    end

    def event_params
      params.require(:event).permit('title', 'description', 'starttime', 'endtime', 'all_day', 'period', 'frequency', 'commit_button')
    end

    def determine_event_type
      if params[:event][:period] == "Does not repeat"
        @event = Event.new(event_params)
      else
        @event = EventSeries.new(event_params)
      end
    end

    def make_time_from_minute_and_day_delta(event_time)
      params[:minute_delta].to_i.minutes.from_now((params[:day_delta].to_i).days.from_now(event_time))
    end
  end
end
